@@ -36,7 +36,7 @@ import rasaeco.model
 import rasaeco.template
 
 
-def _render_ontology(
+def _render_ontology_html(
     ontology: rasaeco.model.Ontology,
     scenarios_dir: pathlib.Path,
     path_map: Mapping[str, pathlib.Path],
@@ -82,10 +82,16 @@ def _render_ontology(
             )
         )
 
+    dataset = Dataset(nodes=nodes, edges=edges)
+
+    ##
+    # Render to HTML
+    ##
+
     pth = scenarios_dir / "ontology.html"
 
     ontology_html = rasaeco.template.ONTOLOGY_HTML_TPL.render(
-        dataset=json.dumps(Dataset(nodes=nodes, edges=edges), indent=2)
+        dataset=json.dumps(dataset, indent=2)
     )
 
     try:
@@ -93,6 +99,26 @@ def _render_ontology(
     except Exception as exception:
         return [f"Failed to write the ontology to {pth}: {exception}"]
 
+    ##
+    # Render to DOT
+    ##
+
+    pth = scenarios_dir / "ontology.dot"
+
+    for node in nodes:
+        assert (
+            " " not in node["url"]
+        ), f"Unexpected double quote in the URL of a node: {node['url']}"
+
+    ontology_dot = rasaeco.template.ONTOLOGY_DOT_TPL.render(
+        dataset=dataset, dumps=json.dumps
+    )
+
+    try:
+        pth.write_text(ontology_dot, encoding="utf-8")
+    except Exception as exception:
+        return [f"Failed to write the ontology to {pth}: {exception}"]
+
     return []
 
 
@@ -360,6 +386,7 @@ def _render_scenario_to_xml(
         root.iter("acceptanceref"),
         root.iter("phase"),
         root.iter("level"),
+        root.iter("scenarioref"),
     ):
         if "name" not in element.attrib:
             errors.append(
@@ -425,7 +452,7 @@ def _parse_reference_element(element: ET.Element) -> Tuple[Optional[str], str]:
     """Extract the scenario identifier and the name from a reference element."""
     name_attribute = element.attrib["name"]
     if "#" in name_attribute:
-        scenario_id, name = name_attribute.split("#")
+        scenario_id, name = name_attribute.split("#", 1)
         return scenario_id, name
     else:
         return None, name_attribute
@@ -448,6 +475,10 @@ def _validate_references(
 
     root = ET.fromstring(text)
 
+    ##
+    # Validate the references for different tags
+    ##
+
     class SetGetterForScenario(Protocol):
         def __call__(self, scenario_id: str) -> Set[str]:
             ...
@@ -484,26 +515,37 @@ def _validate_references(
 
         return errors
 
-    return (
+    errors = []  # type: List[str]
+
+    errors.extend(
         validate_references_for_tag(
             reference_tag="modelref",
             set_getter_for_scenario=lambda scenario_id: ontology.scenario_map[
                 scenario_id
             ].definitions.model_set,
         )
-        + validate_references_for_tag(
+    )
+
+    errors.extend(
+        validate_references_for_tag(
             reference_tag="ref",
             set_getter_for_scenario=lambda scenario_id: ontology.scenario_map[
                 scenario_id
             ].definitions.def_set,
         )
-        + validate_references_for_tag(
+    )
+
+    errors.extend(
+        validate_references_for_tag(
             reference_tag="testref",
             set_getter_for_scenario=lambda scenario_id: ontology.scenario_map[
                 scenario_id
             ].definitions.test_set,
         )
-        + validate_references_for_tag(
+    )
+
+    errors.extend(
+        validate_references_for_tag(
             reference_tag="acceptanceref",
             set_getter_for_scenario=lambda scenario_id: ontology.scenario_map[
                 scenario_id
@@ -511,6 +553,21 @@ def _validate_references(
         )
     )
 
+    ##
+    # Validate the scenario references as a special case
+    ##
+
+    for element in root.iter("scenarioref"):
+        scenario_id = element.attrib["name"]
+
+        if scenario_id not in ontology.scenario_map:
+            errors.append(
+                f"The scenarioref is invalid: {_element_to_str(element)}; "
+                f"the scenario with the identifier {scenario_id} does not exist."
+            )
+
+    return errors
+
 
 _INFLECT_ENGINE = inflect.engine()
 
@@ -612,8 +669,7 @@ def _render_scenario(
             element.text = link_text
 
     @icontract.require(
-        lambda reference_tag: reference_tag
-        in ["modelref", "ref", "testref", "acceptanceref"]
+        lambda reference_tag: reference_tag in ["modelref", "testref", "acceptanceref"]
     )
     def convert_references_to_html(reference_tag: str) -> None:
         """Convert the reference tags to proper HTML."""
@@ -653,6 +709,25 @@ def _render_scenario(
     convert_references_to_html(reference_tag="testref")
     convert_references_to_html(reference_tag="acceptanceref")
 
+    ##
+    # Convert <scenarioref>s to links
+    ##
+
+    for element in root.iter("scenarioref"):
+        scenario_id = element.attrib["name"]
+
+        target_scenario = ontology.scenario_map[scenario_id]
+
+        href = _html_path(
+            scenario_path=rel_pth_to_scenario_dir / target_scenario.relative_path
+        ).as_posix()
+
+        element.tag = "a"
+        element.attrib = {"href": href, "class": "scenarioref"}
+
+        if len(element) == 0 and not element.text:
+            element.text = f'"{target_scenario.title}"'
+
     ##
     # Replace <phase> tags with proper HTML
     ##
@@ -1189,7 +1264,9 @@ def once(scenarios_dir: pathlib.Path) -> List[str]:
         plot_pth = path_map[scenario.identifier].parent / "volumetric.png"
         _render_volumetric_plot(plot_path=plot_pth, scenario=scenario)
 
-    _render_ontology(ontology=ontology, scenarios_dir=scenarios_dir, path_map=path_map)
+    _render_ontology_html(
+        ontology=ontology, scenarios_dir=scenarios_dir, path_map=path_map
+    )
 
     for scenario in scenarios:
         pth = path_map[scenario.identifier]
